;  Test SWCHB all values
;  Written by Omegamatrix

      processor 6502

VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
COLUBK  =  $09
RESP0   =  $10
RESP1   =  $11
GRP0    =  $1B
GRP1    =  $1C
HMP0    =  $20
HMP1    =  $21
VDELP0  =  $25
VDELP1  =  $26
HMOVE   =  $2A
INTIM   =  $0284
TIM64T  =  $0296
SWCHB   =  $0282
SWBCNT  =  $0283

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      RAM
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG.U Variables
       ORG $80

counterForIO       ds 1
writeVariable      ds 1
badFlag            ds 1
loopCount          ds 1
tempOne            ds 1
tempSaveInputs     ds 1
tempSaveOutputs    ds 1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      INITIALIZATION
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG Code
       ORG $F000

START:
    sei
    cld
    ldx    #0
    txa
.loopClear:
    dex
    txs
    pha
    bne    .loopClear
    
    lda    #02                   ; keep electron beam off for this long code check
    sta    VBLANK

    sta    WSYNC
;---------------------------------------
    lda    #$0F                  ;2  @2     white
    sta    COLUP0                ;3  @5
    sta    COLUP1                ;3  @8
    sta    VDELP0                ;3  @11    delay
    sta    VDELP1                ;3  @14
    lda    #$03                  ;2  @16
    sta    NUSIZ0                ;3  @29    3 copies close
    sta    NUSIZ1                ;3  @22
    lda    #$80                  ;2  @24    right 8
    sta    HMP0                  ;3  @27
    lda    #$90                  ;2  @29    right 7
    sta    HMP1                  ;3  @32
    nop                          ;2  @34
    sta    RESP0                 ;3  @37
    sta    RESP1                 ;3  @40

    sta    WSYNC
;---------------------------------------
    sta    HMOVE
    
    

;hammer VSYNC a bunch of times, stop Stella from auto-detecting "PAL" format
    ldy    #40                 
.loopForceStellaToNTSC:
    lda    #$0E
.loopVsync1:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync1
    dey
    bpl    .loopForceStellaToNTSC
    
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      TEST LOGIC
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    
    
    lda    counterForIO          ; initial value is $00
    
.outerLoop:
    sta    SWBCNT                ; change I/O
    eor    #$FF                  ; flip bits, because input for the Atari is "0" and not "1"
    and    SWCHB                 
    sta    tempSaveInputs        ; and keep only the inputs...
    
    lda    writeVariable
.innerLoop:
    sta    SWCHB                 ; write a new value to the switch
    
    lda    writeVariable
    and    counterForIO
    sta    tempSaveOutputs       ; save expected value for outputs
    
;check inputs
    lda    counterForIO
    eor    #$FF
    and    SWCHB
    cmp    tempSaveInputs        ; have the inputs have changed?
    bne    .setBadFlag           ; if yes, break the loop
    
;check outputs
    lda    counterForIO
    and    SWCHB                 ; check if the output matches
    cmp    tempSaveOutputs
    bne    .setBadFlag           ; if yes, break the loop
    
;prep for next loop
    inc    writeVariable
    lda    writeVariable         ; all values 0-255 are tested against every I/O combination
    bne    .innerLoop
    inc    counterForIO          ; scroll through every possible I/O combination
    lda    counterForIO
    bne    .outerLoop
    beq    DoVsync               ; always branch, everything went fine
    
.setBadFlag:
    lda    #$FF                  ; check failed (at least one of the bits changed its value)
    sta    badFlag               ; the user can check which combination failed (ram $80) for
                                 ; what value (ram $81) by entering Stella's debugger.
                                 
                                 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      VBLANK
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DoVsync:
    lda    #$0E                     ; 3 lines of Vsync
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync
    lda    #$30
    sta    TIM64T

    lda    badFlag
    beq    .good
    ldy    #$44                     ; red screen = write values have changed 
    bne    .doScreenColor           ; always branch

.good:
    ldy    #$A6                     ; blue screen = write values unaffected
.doScreenColor:
    sty    COLUBK


.waitVblank:
    lda    INTIM
    bne    .waitVblank
    sta    WSYNC
;---------------------------------------
    sta    VBLANK                   ; Vblank ends

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      KERNEL
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #93
.loopTop:
    sta    WSYNC
;---------------------------------------
    dey                          ;2  @2
    bne    .loopTop              ;2 @4/5

    ldy    #7
    sty    loopCount
.loopSwchbGfx:
    ldy    loopCount             ;3  @64
    lda    GfxSWCHB6,Y           ;4  @68
    sta    tempOne               ;3  @71    6th
    sta    WSYNC                 ;3
;---------------------------------------
    lda    GfxSWCHB1,Y           ;4  @4     1st
    sta    GRP0                  ;3  @7
    lda    GfxSWCHB2,Y           ;4  @11    2nd
    sta    GRP1                  ;3  @14
    lda    GfxSWCHB3,Y           ;4  @18    3rd
    sta    GRP0                  ;3  @21
    nop                          ;2  @23
    nop                          ;2  @25
    dec    $2E                   ;5  @30
    ldx    GfxSWCHB4,Y           ;4  @34    4th
    lda    GfxSWCHB5,Y           ;4  @38    5th
    ldy    tempOne               ;3  @41    6th
    stx    GRP1                  ;3  @44
    sta    GRP0                  ;3  @47
    sty    GRP1                  ;3  @50
    sta    GRP0                  ;3  @53
    dec    loopCount             ;5  @58
    bpl    .loopSwchbGfx         ;2 @60/61
    lda    #0                    ;2  @62
    sta    GRP0                  ;3  @65
    sta    GRP1                  ;3  @68
    sta    GRP0                  ;3  @71
    sta    GRP1                  ;3  @74


    ldy    #94
.loopBottom:
    sta    WSYNC
;---------------------------------------
    dey                          ;2  @2
    bne    .loopBottom           ;2 @4/5


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;      OVERSCAN
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    #$82
    sta    WSYNC
;---------------------------------------
    sta    VBLANK                   ; turn beam off, start overscan period
    lda    #26
    sta    TIM64T

.waitOverscan:
    lda    INTIM
    bne    .waitOverscan
    jmp    DoVsync



       ORG $FF00

GfxSWCHB1:
    .byte $0F ; |    XXXX| $FF00
    .byte $0F ; |    XXXX| $FF01
    .byte $00 ; |        | $FF02
    .byte $07 ; |     XXX| $FF03
    .byte $0F ; |    XXXX| $FF04
    .byte $0C ; |    XX  | $FF05
    .byte $0F ; |    XXXX| $FF06
    .byte $07 ; |     XXX| $FF07
GfxSWCHB2:
    .byte $C6 ; |XX   XX | $FF08
    .byte $EF ; |XXX XXXX| $FF09
    .byte $6D ; | XX XX X| $FF0A
    .byte $ED ; |XXX XX X| $FF0B
    .byte $CD ; |XX  XX X| $FF0C
    .byte $0C ; |    XX  | $FF0D
    .byte $EC ; |XXX XX  | $FF0E
    .byte $EC ; |XXX XX  | $FF0F
GfxSWCHB3:
    .byte $63 ; | XX   XX| $FF10
    .byte $F7 ; |XXXX XXX| $FF11
    .byte $B6 ; |X XX XX | $FF12
    .byte $B6 ; |X XX XX | $FF13
    .byte $B6 ; |X XX XX | $FF14
    .byte $36 ; |  XX XX | $FF15
    .byte $37 ; |  XX XXX| $FF16
    .byte $33 ; |  XX  XX| $FF17
GfxSWCHB4:
    .byte $E6 ; |XXX  XX | $FF18
    .byte $F6 ; |XXXX XX | $FF19
    .byte $36 ; |  XX XX | $FF1A
    .byte $07 ; |     XXX| $FF1B
    .byte $07 ; |     XXX| $FF1C
    .byte $36 ; |  XX XX | $FF1D
    .byte $F6 ; |XXXX XX | $FF1E
    .byte $E6 ; |XXX  XX | $FF1F
GfxSWCHB5:
    .byte $37 ; |  XX XXX| $FF20
    .byte $37 ; |  XX XXX| $FF21
    .byte $36 ; |  XX XX | $FF22
    .byte $F7 ; |XXXX XXX| $FF23
    .byte $F7 ; |XXXX XXX| $FF24
    .byte $36 ; |  XX XX | $FF25
    .byte $37 ; |  XX XXX| $FF26
    .byte $37 ; |  XX XXX| $FF27
GfxSWCHB6:
    .byte $E0 ; |XXX     | $FF28
    .byte $F0 ; |XXXX    | $FF29
    .byte $30 ; |  XX    | $FF2A
    .byte $E0 ; |XXX     | $FF2B
    .byte $E0 ; |XXX     | $FF2C
    .byte $30 ; |  XX    | $FF2D
    .byte $F0 ; |XXXX    | $FF2E
    .byte $E0 ; |XXX     | $FF2F

       ORG $FFFC

    .word START,START